<?php
/*
Plugin Name: Authorize.net - Simple Donations
Author: Aman Verma
Author URI: https://twitter.com/amanverma217
Plugin URL: https://twitter.com/amanverma217
Description: Accept donations simply with Authorize.net. Easy to use and configure.
Tags: authorize.net donation plugin, donation plugin, authoriz.net, donation widget
Version: 2.3.3
License: GPLv2 or later
*/

use net\authorize\api\contract\v1 as AnetAPI;
use net\authorize\api\controller as AnetController;

define( 'DONATION_FORM_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
define( 'DONATION_FORM_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );

require_once DONATION_FORM_PLUGIN_DIR . 'functions.php';
require_once DONATION_FORM_PLUGIN_DIR . 'donation_list_table.php'; 

function load_authorize_script() { 
	wp_enqueue_script( 'jquery' );
    wp_enqueue_style( 'donation_form_css', plugin_dir_url(__FILE__ ) . 'assets/css/donation_form.css');
	wp_enqueue_script( 'donation_form_js', plugin_dir_url(__FILE__ ) . 'assets/js/donation_form.js');
}
add_action( 'wp_enqueue_scripts', 'load_authorize_script' );

function send_wps_email( $to, $subject, $body, $headers = '', $attachments = array() ) {
	$headers .= 'MIME-Version: 1.0' . "\r\n";
	$headers .= 'Content-Type: text/html; charset=UTF-8' . "\r\n";
	$headers .= 'Content-Transfer-Encoding: 8bit' . "\r\n";
	$headers .= 'X-Mailer: PHP/' . phpversion() . "\r\n";
	$headers .= 'X-Priority: 1 (Highest)' . "\n";
	$headers .= 'X-MSMail-Priority: High' . "\n";
	$headers .= 'Importance: High' . "\n";

	// If you want to attach a file to mail, Please give file path (Not url), for eg. /home/html/uploads/file.jpg
	wp_mail( $to, $subject, $body, $headers, $attachments );
}
		
add_shortcode('wds_donate', 'fn_wds_donate');
function fn_wds_donate()
{
	
	$valid	  = true;
	$msg  	  = '';
	$donation  = false;
	$html 	  = '';
	
	if( isset( $_POST['wds_donate'])):
		 
		if( $_POST['donor_firstname'] != ''){
			$donor_firstname = $_POST['donor_firstname'];
		} else {
			$valid = false;
			$msg.= 'First Name is required </br>';
		}

		if( $_POST['donor_lastname'] != ''){
			$donor_lastname = $_POST['donor_lastname'];
		} else {
			$valid = false;
			$msg.= 'Last Name is required </br>';
		}

		if( $_POST['donor_email'] != ''){
			
			$donor_email = $_POST['donor_email'];
			
			if( preg_match("/([\w\-]+\@[\w\-]+\.[\w\-]+)/" , $donor_email)){
				
			} else {
				$valid = false;
				$msg.= 'Invalid email format </br>';
			}
			
		} else {
			$valid = false;
			$msg.= 'E-mail is required </br>';
		}

		if( $_POST['donation_amount'] != '') {
			
			$donation_amount = $_POST['donation_amount'];
			
			if( (is_numeric($donation_amount)) && ( (strlen($donation_amount) > '1') || (strlen($donation_amount) == '1')) ){
			
			} else {
				$valid = false;
				$msg.= 'Amount cannot be less then $1</br>';
			}
			
		} else {
			$valid = false;
			$msg.= 'Amount is required </br>';
		}


		if( $_POST['donor_card_number'] != ''){
			
			$donor_card_number = $_POST['donor_card_number'];
			
		    if( (is_numeric($donor_card_number)) /*&& (strlen($donor_card_number) > '15')*/ ){
				
			} else {
				$valid = false;
				$msg.= 'Please enter valid Card Number</br>';
			}
			
		} else {
			$valid = false;
			$msg.= 'Card Number is required </br>';
		}
		


		if( $_POST['donor_cvv'] != ''){
			
			$donor_cvv = $_POST['donor_cvv'];
			
			if( (is_numeric($donor_cvv)) && ( strlen($donor_cvv) == '3' || strlen($donor_cvv) == '4' ) ){
				
			} else {
				$valid = false;
				$msg.= 'Please enter valid CVV </br>';
			}
			
		} else {
			$valid = false;
			$msg.= 'CVV is required </br>';
		}


		if( $_POST['donor_card_expiry'] != ''){
			
			$donor_card_expiry = $_POST['donor_card_expiry'];
			
			if( (is_numeric($donor_cvv)) /*&& (strlen($donor_cvv) == '3')*/ ){
			
			} else {
				$valid = false;
				$msg.= 'Please enter valid Card Expiry Date</br>';
			}
			
		} else {
			$valid = false;
			$msg.= 'Card Expiry Date is required </br>';
		}


		if( $valid ){
			
			$donor_firstname;
			$donor_lastname;
			$donor_email;
			$donation_amount;
			$donor_card_number;
			$donor_cvv;
			$donor_card_expiry;
			$auth_login_id        = get_option('wds_donation_login_id');
			$auth_transaction_key = get_option('wds_donation_transaction_key');
			$auth_mode            = get_option('wds_donation_mode');
			//$flag_admin 		  = get_option('wds_admin_notification');
			//$flag_donor 		  = get_option('wds_donor_notification');
			$processor_description = get_option('wds_processor_description');
			
			$month  = substr($donor_card_expiry, 0, 2);
			$year   = substr($donor_card_expiry, 2, 4);
			
			if( sipost('monthly_donation') ) {
				  
				define("MERCHANT_LOGIN_ID", $auth_login_id);
				define("MERCHANT_TRANSACTION_KEY", $auth_transaction_key);
					
				require DONATION_FORM_PLUGIN_DIR . 'vendor/autoload.php'; 
				  
				/* Create a merchantAuthenticationType object with authentication details retrieved from the constants file */
				$merchantAuthentication = new AnetAPI\MerchantAuthenticationType();
				$merchantAuthentication->setName(MERCHANT_LOGIN_ID);
				$merchantAuthentication->setTransactionKey(MERCHANT_TRANSACTION_KEY);
				  
				// Set the transaction's refId
				$refId = 'ref' . time();
				
				// Subscription Type Info
				$subscription = new AnetAPI\ARBSubscriptionType();
				$subscription->setName("Donation");
				
				$interval = new AnetAPI\PaymentScheduleType\IntervalAType();
				$interval->setLength("1");
				$interval->setUnit("months");
				
				$paymentSchedule = new AnetAPI\PaymentScheduleType();
				$paymentSchedule->setInterval($interval);
				$paymentSchedule->setStartDate(new DateTime( date('Y-m-d', strtotime( date('Y-m-d') . " +1 days") ) ) );
				$paymentSchedule->setTotalOccurrences("12");
				$paymentSchedule->setTrialOccurrences("0");
				 
				$subscription->setPaymentSchedule($paymentSchedule);
				$subscription->setAmount( $donation_amount );
				$subscription->setTrialAmount("0.00");
				 
				$creditCard = new AnetAPI\CreditCardType();
				$creditCard->setCardNumber( $donor_card_number );
				$creditCard->setExpirationDate("20".$year."-".$month);
				 
				$payment = new AnetAPI\PaymentType();
				$payment->setCreditCard($creditCard);
				$subscription->setPayment($payment); 
				
				$order = new AnetAPI\OrderType();
				$order->setInvoiceNumber(rand(1111111,9999999));        
				$order->setDescription("Tih Missons Donations"); 
				$subscription->setOrder($order);  
				
				$billTo = new AnetAPI\NameAndAddressType();
				$billTo->setFirstName($donor_firstname);
				$billTo->setLastName($donor_lastname);
				
				$subscription->setBillTo($billTo);
				 
				$request = new AnetAPI\ARBCreateSubscriptionRequest();
				$request->setmerchantAuthentication($merchantAuthentication);
				$request->setRefId($refId);
				$request->setSubscription($subscription); 
				
				$controller = new AnetController\ARBCreateSubscriptionController($request);	 
				
				if( $auth_mode == "live" ) {
					$response = $controller->executeWithApiResponse( \net\authorize\api\constants\ANetEnvironment::PRODUCTION); 
				} else {
					$response = $controller->executeWithApiResponse( \net\authorize\api\constants\ANetEnvironment::SANDBOX);
				}
				 
				if ( ( $response != null) && ($response->getMessages()->getResultCode() == "Ok" ) ) {
					
					$post = array(
						'post_type'    => 'wds_donation',
						'post_title'   => 'Donation - '. sanitize_text_field($_POST['donor_firstname']). ' '. sanitize_text_field($_POST['donor_lastname']),
						'post_status'  => 'publish',
						'post_author'  => 1,
					);
					
					$post_id = wp_insert_post( $post );
					
					// subscription ID
					$transaction_id = $response->getSubscriptionId();
					$approval_code	= $response->getMessages()->getMessage();
					$approval_code	= ($approval_code[0]->getCode());
					
					add_post_meta($post_id, 'payment_mode', "monthly", true);
					add_post_meta($post_id, 'donor_firstname', sanitize_text_field($_POST['donor_firstname']), true);
					add_post_meta($post_id, 'donor_lastname', sanitize_text_field($_POST['donor_lastname']), true);
					add_post_meta($post_id, 'donor_email', sanitize_text_field($_POST['donor_email']), true);
					add_post_meta($post_id, 'donation_amount', sanitize_text_field($_POST['donation_amount']), true);
					add_post_meta($post_id, 'transaction_id', sanitize_text_field($transaction_id), true);
					add_post_meta($post_id, 'last_4', sanitize_text_field($last_4), true);
					add_post_meta($post_id, 'approval_code', sanitize_text_field($approval_code), true);
					
					send_wps_email( sanitize_text_field( $_POST['donor_email'] ), 'Thank you for your donation!', "Treasures in Heaven Missions Inc
<br/> 
Company EIN: 86-2532988
<br/> 
<br/> 
Treasures in Heaven Missions would like to thank you for your donation of $" . sanitize_text_field( $_POST['donation_amount'] ) . " on " . 	date( get_option( 'date_format' ), current_time( 'timestamp' ) ) . ".
" );
					
					$donation = true;
					
				} else {
					
			        $errorMessages = $response->getMessages()->getMessage();
					$msg .= $errorMessages[0]->getText();
	
				}  
				
			} else {
			
				if( get_option('wds_donation_mode') == "live" ){
					$post_url = "https://secure.authorize.net/gateway/transact.dll";
				} else {
					$post_url = "https://test.authorize.net/gateway/transact.dll";
				}

				$post_values = array(
					
					"x_login"			=> $auth_login_id,
					"x_tran_key"		=> $auth_transaction_key,
	
					"x_version"			=> "3.1",
					"x_delim_data"		=> "TRUE",
					"x_delim_char"		=> "|",
					"x_relay_response"	=> "FALSE",
	
					"x_type"			=> "AUTH_CAPTURE",
					"x_method"			=> "CC",
					"x_card_num"		=> $donor_card_number,
					"x_card_code"		=> $donor_cvv,
					"x_exp_date"		=> $donor_card_expiry,
	
					"x_amount"			=> $donation_amount,
					"x_description"		=> $processor_description,
	
					"x_first_name"		=> $donor_firstname,
					"x_last_name"		=> $donor_lastname,
					"x_email"			=> $donor_email,
					"x_address"			=> "",
					"x_state"			=> "",
					"x_zip"				=> ""
				);
	
				$post_string = "";
				foreach( $post_values as $key => $value )
					{ $post_string .= "$key=" . urlencode( $value ) . "&"; }
				$post_string = rtrim( $post_string, "& " );


				$request = curl_init($post_url); 
					curl_setopt($request, CURLOPT_HEADER, 0); 
					curl_setopt($request, CURLOPT_RETURNTRANSFER, 1); 
					curl_setopt($request, CURLOPT_POSTFIELDS, $post_string); 
					curl_setopt($request, CURLOPT_SSL_VERIFYPEER, FALSE);
					$post_response = curl_exec($request); 
					// Connection to Authorize.net			
				curl_close ($request); // close curl object

				
				$response_array = explode($post_values["x_delim_char"],$post_response);
				$transaction_id = $response_array[6];
				$last_4         = $response_array[50];
				$approval_code  = $response_array[4];
				
				if($response_array[0] =='1'){
				
					$post = array(
						'post_type'    => 'wds_donation',
						'post_title'   => 'Donation - '. sanitize_text_field($_POST['donor_firstname']). ' '. sanitize_text_field($_POST['donor_lastname']),
						'post_status'  => 'publish',
						'post_author'  => 1,
					);
					
					$post_id = wp_insert_post( $post );
	                
	                add_post_meta($post_id, 'payment_mode', "one_time", true);
					add_post_meta($post_id, 'donor_firstname', sanitize_text_field($_POST['donor_firstname']), true);
					add_post_meta($post_id, 'donor_lastname', sanitize_text_field($_POST['donor_lastname']), true);
					add_post_meta($post_id, 'donor_email', sanitize_text_field($_POST['donor_email']), true);
					add_post_meta($post_id, 'donation_amount', sanitize_text_field($_POST['donation_amount']), true);
					add_post_meta($post_id, 'transaction_id', sanitize_text_field($transaction_id), true);
					add_post_meta($post_id, 'last_4', sanitize_text_field($last_4), true);
					add_post_meta($post_id, 'approval_code', sanitize_text_field($approval_code), true);
					
					send_wps_email( sanitize_text_field($_POST['donor_email']), 'Thank you for your donation!', "Treasures in Heaven Missions Inc
<br/> 
Company EIN: 86-2532988
<br/> 
<br/> 
Treasures in Heaven Missions would like to thank you for your donation of $" . sanitize_text_field($_POST['donation_amount']) . " on " . 	date( get_option( 'date_format' ), current_time( 'timestamp' ) ) . ".
" );
						
					$donation = true;
				} else if($response_array[0] =='2'){
					$msg .= $response_array[3] ;
	
				} else{
					$msg .= $response_array[3];
				}
				
			} 
			
		} else {	
		
		}
	endif;
	
	if($msg != ''){
		$html .= '<div style="border:double 3px #731D1D; padding:5px;">
				'.$msg.'
				</div>';
	}
	
	if($donation){
		$html .= '<div style="border:double 3px #008000; padding:5px;">' . get_option('wds_thankyou_message') . '</div>';
	
		
	}
	else{
	
	$html .= '<form method="post" class="frmd row">
				<div class="dontaion-price-box-section">        	  
					<div class="dontaion-price-box" data-value="20" data-msg="$20 Donation">
						<span class="price">$20</span>
					</div>
					<div class="dontaion-price-box" data-value="40" data-msg="$40 Donation">
						<span class="price">$40</span>
					</div>
					<div class="dontaion-price-box" data-value="60" data-msg="$60 Donation">
						<span class="price">$60</span>
					</div>
					<div class="dontaion-price-box" data-value="80" data-msg="$80 Donation">
						<span class="price">$80</span>
					</div>
					<div class="dontaion-price-box" data-value="100" data-msg="$100 Donation">
						<span class="price">$100</span>
					</div>           
				</div>
				<div class="vc_column_container col-md-6 mb-2">
					<label>First Name</label></br> 
					<input type="text" class="form-control" placeholder="First Name"  name="donor_firstname" id="donor_firstname"> 
				</div>
				<div class="vc_column_container col-md-6 mb-2">
					<label>Last Name</label></br> 
					<input type="text" class="form-control" placeholder="Last Name"  name="donor_lastname" id="donor_lastname"> 
				</div>
			    <div class="vc_column_container col-md-6 mb-2">
					<label>Email</label></br> 
					<input type="text" class="form-control" placeholder="Email" name="donor_email" id="donor_email">
				</div> 
				<div class="vc_column_container col-md-6 mb-2">
					<label>Amount</label></br> 
					<input type="text" class="form-control" placeholder="Amount (USD)"  name="donation_amount" id="donation_amount" required> 
				</div>
		    	<div class="vc_column_container col-md-12 mb-2">
					<label>Card Number</label><br/> 
					<input type="text" maxlength="25" class="form-control" name="donor_card_number" id="donor_card_number" placeholder="xxxxxxxxxxxxxxxx"> 
				</div>
				<div class="vc_column_container col-md-6 mb-2">
					<label>CVV</label> <br/>
					<input type="text" maxlength="4"  class="form-control" name="donor_cvv" id="cvv" placeholder="xxx"> 
				</div>
			    <div class="vc_column_container col-md-6 mb-2">
					<label>Card Expiry (mmyy)</label></br> 
					<input type="text" class="form-control" maxlength="4"  name="donor_card_expiry" id="donor_card_expiry" placeholder="mmyy"> 
				</div>
				<div class="vc_column_container col-md-12 mb-2 monthly_donation_field">
					<input type="checkbox" name="monthly_donation" value="1">  I would like to make this a monthly donation
				</div>
				<div class="vc_column_container col-md-6 mb-2">
					<input type="submit" name="wds_donate" class="wds_donate" value="Donate Now"> 
				</div>
			</form>';
	
	}
	
	return $html;
}

function wds_donation_type() {

	$labels = array(
		'name'                => _x( 'Donations', 'Post Type General Name', 'wds_donation' ),
		'singular_name'       => _x( 'Donation', 'Post Type Singular Name', 'wds_donation' ),
		'menu_name'           => __( 'Donation ', 'wds_donation' ),
		'parent_item_colon'   => __( 'Parent Donation', 'wds_donation' ),
		'all_items'           => __( 'All Donation', 'wds_donation' ),
		'view_item'           => __( 'View Donation', 'wds_donation' ),
		'add_new_item'        => __( 'Add New Donation', 'wds_donation' ),
		'add_new'             => __( 'Add New', 'wds_donation' ),
		'edit_item'           => __( 'Edit Donation', 'wds_donation' ),
		'update_item'         => __( 'Update Donation', 'wds_donation' ),
		'search_items'        => __( 'Search Donation', 'wds_donation' ),
		'not_found'           => __( 'Not found', 'wds_donation' ),
		'not_found_in_trash'  => __( 'Not found in Trash', 'wds_donation' ),
	);
	$args = array(
		'label'               => __( 'wds_donation', 'wds_donation' ),
		'description'         => __( 'list of donations', 'wds_donation' ),
		'labels'              => $labels,
		'supports'            => array('title', 'custom-fields' ),
		'hierarchical'        => false,
		'public'              => true,
		'show_ui'             => true,
		'show_in_menu'        => true,
		'show_in_nav_menus'   => true,
		'show_in_admin_bar'   => true,
		'menu_position'       => 5,
		'can_export'          => true,
		'has_archive'         => false,
		'exclude_from_search' => true,
		'publicly_queryable'  => true,
		'capability_type'     => 'post',
	);
	register_post_type( 'wds_donation', $args );

}

add_action( 'init', 'wds_donation_type', 0 );

add_action('admin_menu', 'wds_create_menu');

function wds_create_menu() {

	add_menu_page('Donation Settings', 'Donation Settings', 'administrator', 'donation-settings-page', 'donation_settings_page');
	add_menu_page( 'Donation List', 'Donation List', 'administrator', 'donation_list', 'donation_list' );
    add_action( 'admin_init', 'donation_register_mysettings' );
}


function donation_register_mysettings() {

	register_setting( 'wds-settings-group', 'wds_donation_login_id' );
	register_setting( 'wds-settings-group', 'wds_donation_transaction_key' );
	register_setting( 'wds-settings-group', 'wds_donation_mode' );
	register_setting( 'wds-settings-group', 'wds_thankyou_message' );
	register_setting( 'wds-settings-group', 'wds_processor_description' );

}
 
add_action( 'load-toplevel_page_donation_list', 'donation_add_screen_options' );

function donation_add_screen_options() {

	$option = 'per_page';

	$args = array(
		'label'   => 'Donation',
		'default' => 20,
		'option'  => 'donation_list_per_page',
	);

	add_screen_option( $option, $args );
}

add_filter( 'set-screen-option', 'donation_set_option', 10, 3 );

function donation_set_option( $status, $option, $value ) {

	if ( 'donation_list_per_page' == $option ) {
		return $value;
	}

	return $status;
}

function donation_list() {

	global $wpdb2;

	$donation_list = new Donation_List_Page();

	$donation_list->prepare_items();

	?>
	<div class="wrap">

		<div style="clear:both;"></div>

		<div id="icon-users" class="icon32"><br/></div>
		<h2>Donation List</h2>

		 <div style="clear:both;"></div>

		<form id="images_table-filter" method="get">
			<input type="hidden" name="page" value="<?php echo $_REQUEST['page']; ?>" />
			<?php $donation_list->display(); ?>
		</form>

	</div>
	<?php

}

function donation_settings_page() {
?>
<div class="wrap">
<h2>Donation Settings</h2>

<?php
   
if( isset($_GET['settings-updated']) && $_GET['settings-updated'] == 'true'):
   echo    '<div id="setting-error-settings_updated" class="updated settings-error"> 
<p><strong>Settings saved.</strong></p></div>';
endif;
?>

<form method="post" action="options.php">
    <?php settings_fields( 'wds-settings-group' ); ?>
    <?php do_settings_sections( 'wds-settings-group' ); ?>
    <table class="form-table">
        <tr valign="top">
	    <th scope="row">Authorize.net Login ID</th>
        <td><input type="text" style="width:50%" name="wds_donation_login_id" value="<?php echo get_option('wds_donation_login_id'); ?>" placeholder="API Login ID" /></td>
        </tr>
		
        <tr valign="top">
        <th scope="row">Authorize.net Transaction Key</th>
        <td><input type="text" style="width:50%" name="wds_donation_transaction_key" value="<?php echo get_option('wds_donation_transaction_key'); ?>" placeholder="API Transaction Key" /></td>
        </tr>
		
		<tr valign="top">
        <th scope="row">Mode(Live/Test Sandbox)</th>
        <td><select name="wds_donation_mode" />
				<option value="live" <?php if( get_option('wds_donation_mode') == "live" ): echo 'selected'; endif;?> >Live</option>
				<option value="test" <?php if( get_option('wds_donation_mode') == "test" ): echo 'selected'; endif;?> >Test/Sandbox</option>
			</select></td>
		</tr>

		<tr valign="top">
        <th scope="row">Thank You Message</th>
        <td><input type="text" style="width:50%" name="wds_thankyou_message" value="<?php echo get_option('wds_thankyou_message'); ?>" placeholder="Thank you message visible to Donor after donation" /></td>
        </tr>
		
		<tr valign="top">
        <th scope="row">Processor Description</th>
        <td><input type="text" style="width:50%" name="wds_processor_description" value="<?php echo get_option('wds_processor_description'); ?>" /></td>
        </tr>
       
    </table>
    
    <?php submit_button(); ?>

</form>		
<p style="font-weight:bold">Use shortcode [wds_donate]</p>
</div><?php
}

function donation_add_meta_box()
{
   $screens = array( 'wds_donation' );

   foreach ( $screens as $screen ) {

      add_meta_box(  'myplugin_sectionid', __( 'Donation Information', 'myplugin_textdomain' ),'donation_meta_box_callback', $screen, 'normal','high' );
   }
}

add_action( 'add_meta_boxes', 'donation_add_meta_box' );

function donation_meta_box_callback($post)
{

?>

   <ul>
   <li>
   <div class="html-label">First Name</div>

   <div class="html-field form-group" ><input type="text" name="donor_firstname" id="donor_firstname" disabled value="<?php echo esc_attr(get_post_meta( $post->ID, 'donor_firstname', true ));?>"></div>
   </li>

   <li>
   <div class="html-label">Last Name</div>

   <div class="html-field form-group"><input type="text" name="donor_lastname" id="donor_last_name" disabled value="<?php echo esc_attr(get_post_meta( $post->ID, 'donor_lastname', true ));?>"></div>
   </li>

   <li>
   <div class="html-label">E-mail</div>

   <div class="html-field form-group"><input type="text" name="donor_email" id="donor_email" disabled value="<?php echo esc_attr(get_post_meta( $post->ID, 'donor_email', true ));?>"></div>
   </li>

   <li>
   <div class="html-label">Amount (USD)</div>

   <div class="html-field form-group"><input type="text" name="donation_amount" id="donation_amount" disabled value="<?php echo esc_attr(get_post_meta( $post->ID, 'donation_amount', true ));?>"></div>
   </li>

   <li>
   <div class="html-label">Transaction ID</div>

   <div class="html-field form-group"><input type="text" name="transaction_id" id="transaction_id" disabled value="<?php echo esc_attr(get_post_meta( $post->ID, 'transaction_id', true ));?>"></div>
   </li>

   <li>
   <div class="html-label">Last 4</div>

   <div class="html-field form-group"><input type="text" name="last_4" id="last_4" disabled value="<?php echo esc_attr(get_post_meta( $post->ID, 'last_4', true ));?>"></div>
   </li>

   <li>
   <div class="html-label">ApprovalCode</div>

   <div class="html-field form-group"><input type="text" name="approval_code" id="approval_code" disabled value="<?php echo esc_attr(get_post_meta( $post->ID, 'approval_code', true ));?>"></div>
   </li>
   </ul>


<?php  

}

// Add Coloum in donation list table
add_filter( 'manage_wds_donation_posts_columns', 'wds_donation_filter_posts_columns' );

function wds_donation_filter_posts_columns( $columns ) {
    
  $columns['email']          = __( 'Email' );
  $columns['amount']         = __( 'Amount' );
  $columns['transaction_id'] = __( 'Transaction ID' );
  $columns['is_monthly']     = __( 'Is Monthly ?' );
  $columns['status']         = __( 'Status' ); 
  
  return $columns;
}

// Set Coloum in donation list table
add_action( 'manage_wds_donation_posts_custom_column', 'wds_donation_column', 10, 2);

function wds_donation_column( $column, $post_id ) {
    
    // Email column
    if ( 'email' === $column ) {
        echo get_post_meta( $post_id, 'donor_email', true );
    }
  
    // Amount column
    if ( 'amount' === $column ) {
    
        $amount = "0.00";
    
	    if( get_post_meta( $post_id, 'donation_amount', true ) ) {
		    $donation_amount = get_post_meta( $post_id, 'donation_amount', true );
		    $amount = number_format( $donation_amount, 2 );
	    }  
      
        echo "$" . $amount;
    }
  
     // transaction_id column
    if ( 'transaction_id' === $column ) {
   
        echo get_post_meta( $post_id, 'transaction_id', true );
    
    }
  
    // transaction_id column
    if ( 'transaction_id' === $column ) {
   
        echo get_post_meta( $post_id, 'transaction_id', true );
    
    }
    
    if ( 'is_monthly' === $column ) {
        
        $payment_type = "No";
        
        if( get_post_meta( $post_id, 'payment_mode', true ) == 'monthly' ) {
        	$payment_type = "Yes";
        }    
        
        echo $payment_type;
    }
    	
    if ( 'status' === $column ) {
        
        $payment_status = "Fail";
		if( get_post_meta( $post_id, 'transaction_id', true ) ) {
			$payment_status = "Success";
		}	
        
        echo $payment_status;
    }
    
  
}

add_action( 'restrict_manage_posts', 'wds_donation_manage_posts' );

function wds_donation_manage_posts() {
    
    global $typenow;
     
   if ($typenow=='wds_donation') {
       
        //check if our select has been sent and if so, is it set to "Currently Due"
        if ( isset( $_GET["is_monthly"] ) ){ 
            
            $is_monthly = $_GET["is_monthly"];
             
            if( $is_monthly == "monthly" ) {
            
                $recurring_selected = 'SELECTED';
                
            } else if ( $is_monthly == "one_time" ) {
            
                $one_time_selected = 'SELECTED';    
                
            }
            $selected = $is_monthly == "Recurring" ? ' SELECTED ' : '';
            
        } 
        
        echo "<select id='is_monthly' name='is_monthly'>";
        echo "<option value=''>Payment Type</option>";
        echo "<option value='monthly' ".$recurring_selected.">Monthly</option>";
		echo "<option value='one_time' ".$one_time_selected.">One Time</option>";
        echo "</select>";
    }
}


add_filter('parse_query', 'filter_posts_adminlists');

function filter_posts_adminlists($query) {
    
    global $pagenow;
    
    global $typenow;
    
    if ($pagenow == "edit.php" && $typenow == "wds_donation" ){
        
        $qvars = &$query->query_vars;
        
        if ( isset( $_GET["is_monthly"] ) )  {
            
            if(  $_GET["is_monthly"] == "monthly"  ) {
            
                $meta_value = 'monthly';    
            
            } else {
                
                $meta_value = 'one_time';    
            }
            
            $metaQuery = array(); 
            
            $metaQuery[] = array(
                'key' => 'payment_mode',
                'value' => $meta_value, 
            );
             
            $qvars['meta_query'] = $metaQuery; 
        } 
    }
}

add_action( 'add_meta_boxes', 'payment_histroy_meta_box' );

function payment_histroy_meta_box() {
	
   $screens = array( 'wds_donation' );

   foreach ( $screens as $screen ) {

        add_meta_box(  'payment_history', __( 'Payment History', 'myplugin_textdomain' ),'payment_history_callback', $screen, 'normal','high' );
          
   }
}


function payment_history_callback( $post ) {
    
    if( $post->post_type == 'wds_donation' && get_post_meta( $post->ID, 'payment_mode', true ) == 'monthly' ) {
    
        require DONATION_FORM_PLUGIN_DIR . 'vendor/autoload.php'; 
        
        $subscriptionId = get_post_meta( $post->ID, 'transaction_id', true );
        
        $donation_amount = get_post_meta( $post->ID, 'donation_amount', true );
        
        //define("MERCHANT_LOGIN_ID", get_option('wds_donation_login_id') );
    	//define("MERCHANT_TRANSACTION_KEY", get_option('wds_donation_transaction_key') );
    	
        $merchantAuthentication = new AnetAPI\MerchantAuthenticationType();
    	$merchantAuthentication->setName('7URDj9Wb46jz');
    	$merchantAuthentication->setTransactionKey('5n5LP7d8qWp52E27');
    	  
        // Set the transaction's refId
        $refId = 'ref' . time();
    	
    		
        // Creating the API Request with required parameters
        $request = new AnetAPI\ARBGetSubscriptionRequest();
        $request->setMerchantAuthentication($merchantAuthentication);
        $request->setRefId($refId);
        $request->setSubscriptionId(7740394);
        $request->setIncludeTransactions(true);
    	    
        // Controller
        $controller = new AnetController\ARBGetSubscriptionController($request);
    		
        // Getting the response
        $response = $controller->executeWithApiResponse( \net\authorize\api\constants\ANetEnvironment::SANDBOX);
        
        if( $response->getSubscription()->getArbTransactions() )  {
        
            foreach( $response->getSubscription()->getArbTransactions() as $transaction_result ) {
                
                $transaction_date_time = date("d-m-Y H:i:s", strtotime( $transaction_result->getSubmitTimeUTC()->date ) );
                
                $transaction_time = $transaction_result->getSubmitTimeUTC()->date;
                //print_rr($transaction_result->getSubmitTimeUTC());
                
                echo $transaction_result->getPayNum() .") <b> Transaction ID : </b> " . $transaction_result->getTransId() . ", <b> Amount : $</b>" . $donation_amount . ",<br><br>";
                
            }
        
        } 
    }
    
}